package gov.nih.nlm.nls.lexCheck.CheckCont;
import java.util.*;
import java.io.*;
import gov.nih.nlm.nls.lexCheck.Lib.*;
import gov.nih.nlm.nls.lexCheck.Api.*;
/*****************************************************************************
* This class provides a Java class to check the content of a lexRecord.
* First, it uses lexCheck to check syntax.
* Then, it checks content of a lexical record locally (no cross-reference check)
* Third, It fixes some found issues automatically. 
*
* <p><b>History:</b>
* <ul>
* <li>SCR-39, chlu, 03/28/2013, Add new features to check content
* <li>SCR-44, chlu, 07/12/2013, use HashSet to pass irregExpEuiList
* </ul>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class CheckLexRecord
{
    // private constructor
    private CheckLexRecord()
    {
    }

	public static void CheckLexRecordsFromFile(String inFile, String outFile,
		boolean verbose, HashSet<String> irregExpEuiList)
	{
        try
        {
			// load all lexical records from the input file	
			// This also check the syntax
            Vector<LexRecord> lexRecords 
                = ToJavaObjApi.ToJavaObjsFromTextFile(inFile);
            BufferedWriter out = new BufferedWriter(new OutputStreamWriter(
                new FileOutputStream(outFile), "UTF-8"));

			// check contents on all lexRecords
			CheckLexRecords(lexRecords, out, verbose, irregExpEuiList);
            out.close();
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
	}

	// check contents on all lexRecords locally
	public static void CheckLexRecords(Vector<LexRecord> lexRecords,
		boolean verbose, HashSet<String> irregExpEuiList) throws Exception
	{
		CheckLexRecords(lexRecords, null, verbose, irregExpEuiList);
	}
	// check contents on all lexRecords locally
	public static void CheckLexRecords(Vector<LexRecord> lexRecords,
		BufferedWriter out, boolean verbose, HashSet<String> irregExpEuiList) 
		throws Exception
	{
		int errRecordNo = 0;
		int okRecordNo = 0;
		int recSize = lexRecords.size();

		if(verbose == true)
		{
			System.out.println("===== Check LexRecord Contents =====");
		}
		// go through all Lexical records to check content
		if(lexRecords.size() > 0)
		{
			for(int i = 0; i < recSize; i++)
			{
				LexRecord lexRecord = lexRecords.elementAt(i);

				if(verbose == true)
				{
					System.out.println("--- Checking: " + lexRecord.GetEui()
						+ " ---");
				}

				// check and fix content(s)
				if(CheckLexRecord(lexRecord, irregExpEuiList) == false)
				{
					System.out.println(ErrMsgUtil.GetErrMsg());
					errRecordNo++;
				}
				else
				{
					okRecordNo++;
				}
				// print auto-out fixed records
				if(out != null)
				{
					String text = lexRecord.GetText();
					out.write(text);
				}
			}
		}

		// send out final message 
		System.out.println("----- Total lexRecords checked: " + recSize); 
		// print out err stats
		System.out.println("--- lexRecord has no error: " + okRecordNo);
		System.out.println("--- lexRecord has error(s): " + errRecordNo);
		System.out.println("----- content error type stats -----");
		System.out.println(ErrMsgUtilLexRecord.GetErrStats());
	}

	// Check content of a lexRecord, automatically fixed if possible
	// auto fixes: remove duplicated spVars, abbreviations and acronyms
    public static boolean CheckLexRecord(LexRecord lexRecord, 
		HashSet<String> irregExpEuiList)
    {
		return CheckLexRecord(lexRecord, irregExpEuiList, true);
	}

	/**
	* @param lexRecord 	lexRecord object to be checked content
	* @param irregExpEuiList hashSet for irreg exceptions
	* @param checkEuiFlag boolean flag for checking EUI
	*/
    public static boolean CheckLexRecord(LexRecord lexRecord, 
		HashSet<String> irregExpEuiList, boolean checkEuiFlag)
    {
		// reset errMsg_
        ErrMsgUtilLexRecord.ResetErrMsg();

		// Check current content in this lexRecord
		boolean euiFlag = true;
		if(checkEuiFlag == true)
		{
			euiFlag = CheckEui.CheckContent(lexRecord); // check EUI
		}
		boolean citFlag = CheckCitation.CheckContent(lexRecord); // citation
		boolean svFlag = CheckSpellingVars.CheckContent(lexRecord); // spellVar 
		boolean catFlag = CheckCategory.CheckContent(lexRecord); // check cat
		boolean varFlag = CheckVariants.CheckContent(lexRecord); // Variants
		boolean nomFlag = CheckNominalizations.CheckContent(lexRecord); // Nom
		boolean abbFlag = CheckAbbreviations.CheckContent(lexRecord); // abb
		boolean acrFlag = CheckAcronyms.CheckContent(lexRecord); // acr

		// Check current content with other contents in this lexRecord
		boolean citFlag2 = CheckCitation.CheckContents(lexRecord); // citation
		boolean svFlag2 = CheckSpellingVars.CheckContents(lexRecord); // spelling vars
		boolean varFlag2 = CheckVariants.CheckContents(lexRecord,
			irregExpEuiList); // Variants
		boolean nomFlag2 = CheckNominalizations.CheckContents(lexRecord); // Nom
		boolean abbFlag2 = CheckAbbreviations.CheckContents(lexRecord); // Abbs
		boolean acrFlag2 = CheckAcronyms.CheckContents(lexRecord); // Acronyms

		boolean validFlag = euiFlag && citFlag && svFlag && catFlag
			&& varFlag && nomFlag && abbFlag && acrFlag 
			&& citFlag2 && svFlag2 && varFlag2 && nomFlag2 && abbFlag2 
			&& acrFlag2;
        return validFlag;
    }
    // data member
}
