package gov.nih.nlm.nls.lexCheck.CheckCont;

import java.io.*;
import java.util.*; 

import gov.nih.nlm.nls.lexCheck.Lib.*;

/*****************************************************************************
* This program is used to load key|values table from a file.
* The table is stored in a hash table as Hashtable&lt;String, HashSet&lt;
* String>>.
* It also eliminates duplicates.
* This program could be replaced by database
*
* File Format: key|value-1|value-2|...
* notes: one key could have multiple values from the input file
*
* <p><b>History:</b>
* <ul>
* <li>SCR-40, chlu, 04-22-13, check Lexicon - cross-ref check
* </ul>
*
* @author NLM NLS Development Team
*
* @version    V-2013
****************************************************************************/
public class KeyValuesTable
{
    // private constructor
    private KeyValuesTable()
    {
    }

    // public methods
    /**
    * Get unique values for a key in the key-values table.
    *
    * @param key the key for the mapping values
    * @param keyValuesTable the key-values table
    *
    * @return the unique values for a key in the key-values table.
    */
    public static Vector<String> GetValues(String key, 
        Hashtable<String, HashSet<String>> keyValuesTable)
    {
        HashSet<String> values = keyValuesTable.get(key);
        if(values == null)
        {
            values =  new HashSet<String>();
        }

        return new Vector<String>(values);
    }

    /**
    * Get sorted unique values for a key in the key-values table.
    *
    * @param key the key for the mapping values
    * @param keyValuesTable the key-values table
    *
    * @return the sorted unique values for a key in the key-values table.
    */
    public static Vector<String> GetSortedValues(String key, 
        Hashtable<String, HashSet<String>> keyValuesTable)
    {
        // convert into Vector and sort it
        Vector<String> sortedValues = GetValues(key, keyValuesTable);
        Collections.sort(sortedValues);

        return sortedValues;
    }

    // load data to Hashtable form a file
    // file format: key|value
    /**
    * Get the key-values table by loading all key-values pairs from a file 
    * with options of verbose and show loading lineNum.
	* Format: key-1|key-2|value-1|value-2|...
	* key = key-1|key-2
    *
    * @param inFile the file includes key-values pairs to be loaded
    * @param verbose verbose flag
    * @param lineNum loading status of line number to be displayed
    *
    * @return the key-values table
    */
    public static Hashtable<String, HashSet<String>> GetKey2ValuesTable(
        String inFile, boolean verbose, int lineNum)
    {
        String line = null;
        int lineNo = 0;
        int keyNo = 0;
        int valueNo = 0;

        Hashtable<String, HashSet<String>> keyValues 
            = new Hashtable<String, HashSet<String>>(); 
        try
        {
            BufferedReader in = new BufferedReader(new InputStreamReader(
                new FileInputStream(inFile), "UTF-8"));

            // read in line by line from a file
            while((line = in.readLine()) != null)
            {
                lineNo++;
                // skip the line if it is empty or comments (#)
                if((line.length() > 0) && (line.charAt(0) != '#'))
                {
                    StringTokenizer buf = new StringTokenizer(line, "|");

                    String key1 = buf.nextToken();
                    String key2 = buf.nextToken();
					String key = key1 + LexRecordUtil.FS_STR + key2;

					while(buf.hasMoreTokens() == true)
					{
						String value = buf.nextToken();
						
						// key does not exist
						if(keyValues.containsKey(key) == false)
						{
							HashSet<String> values = new HashSet<String>();
							values.add(value);
							keyValues.put(key, values);
							keyNo++;
							valueNo++;
						}
						else	// key exist
						{
							keyValues.get(key).add(value);
							valueNo++;
						}
					}

                    if((verbose == true)
                    && (lineNo%lineNum == 0))
                    {
                        System.out.println("-- current line: " + lineNo);
                    }
                }
            }

            // print out message
            if(verbose == true)
            {
                System.out.println("-----------------------------");
                System.out.println("--- Total line: " + lineNo);
                System.out.println("--- Total key: " + keyNo);
                System.out.println("--- Total value: " + valueNo);
            }

            // close files
            in.close();
        }
        catch(Exception e)
        {
            System.err.println("** ERR@KeyValuesTable.GetKeyValuesTable (" 
                + lineNo + "): " + line);
            System.err.println("Exception: " + e.toString());
        }

        return keyValues;
    }

    // data members
}
