package gov.nih.nlm.nls.lexCheck.Lib;
/*****************************************************************************
* This class provides methods for various conversion.
*
* <p><b>History:</b>
*
* @author NLM Lexical System Development Team
*
* @version    V-2013
****************************************************************************/
public class Convert
{
    // public method
    /**
    * Test driver for convert methods
    */
    public static void main(String[] args)
    {
        String inStr = "ASCII: A-Z,a-z.\n" + "&,<,>,\",\040\040.\n" 
            + "Space Test";
        if(args.length == 1)
        {
            inStr = args[0];
        }
        System.out.println("-- in: " + inStr);
        System.out.println("-- Name:" + ToNamedEntity(inStr));
        System.out.println("-- Numeric:" + ToNumericEntity(inStr));
        System.out.println("-- ToHtml:" + AsciiToHtml(inStr));
    }
    // public methods
    /**
    * Convert an ASCII input to numerical entity.  For example,
    * "\"" to "&#034;"
    *
    * @param  ascii ascii string for conversion
    *
    * @return  the numerical entity of a ascii
    */
    public static String ToNumericEntity(String ascii)
    {
        // Convert other html entity sequences
        int curPos = 0;
        for(int i = 0; i < ENTITY_LIST.length; i++)
        {
            int length = ENTITY_LIST[i][2].length();
            while((curPos = ascii.indexOf(ENTITY_LIST[i][2], curPos+1)) >= 0)
            {
                ascii = ascii.substring(0, curPos) + ENTITY_LIST[i][0] +
                    ascii.substring(curPos+length);
            }
        }
        return ascii;
    }
    /**
    * Convert an ASCII input to named entity.  For example,
    * "\"" to "&quot;"
    *
    * @param  ascii ascii string for conversion
    *
    * @return  the numerical entity of a ascii
    */
    public static String ToNamedEntity(String ascii)
    {
        // Convert other html entity sequences
        int curPos = 0;
        for(int i = 0; i < ENTITY_LIST.length; i++)
        {
            int length = ENTITY_LIST[i][2].length();
            while((curPos = ascii.indexOf(ENTITY_LIST[i][2], curPos+1)) >= 0)
            {
                ascii = ascii.substring(0, curPos) + ENTITY_LIST[i][1] +
                    ascii.substring(curPos+length);
            }
        }
        return ascii;
    }
    /**
    * Convert an ASCII input to html format.  Tasks include,
    * convert ASCII to numberical entity, such as from "\"" to "&#034;";
    * and change all spaces, carriage return, and tabs.
    *
    * @param  ascii ascii string for conversion
    *
    * @return  the html format of an ascii
    */
    public static String AsciiToHtml(String ascii)
    {
        // Convert other html entity sequences
        int curPos = 0;
        for(int i = 0; i < ENTITY_LIST.length; i++)
        {
            int length = ENTITY_LIST[i][2].length();
            while((curPos = ascii.indexOf(ENTITY_LIST[i][2], curPos+1)) >= 0)
            {
                ascii = ascii.substring(0, curPos) + ENTITY_LIST[i][0] +
                    ascii.substring(curPos+length);
            }
        }
        // Change all space to &nbsp
        while((curPos = ascii.indexOf(" ")) >= 0)
        {
            ascii = ascii.substring(0,curPos) + "&nbsp;"
                + ascii.substring(curPos+1);
        }
 
        // Change all carriage return to &#010;
        while((curPos = ascii.indexOf("\n")) >= 0)
        {
            ascii = ascii.substring(0,curPos) + "&#010;"
                + ascii.substring(curPos+1);
        }
        // Change all tab return to &#009;
        while((curPos = ascii.indexOf("\t")) >= 0)
        {
            ascii = ascii.substring(0,curPos) + "&#009;"
                + ascii.substring(curPos+1);
        }
        return ascii;
    }
    /**
    * Convert an html format input into ASCII string.
    *
    * @param  html html format string for conversion
    *
    * @return  the ascii string
    */
    public static String HtmlToAscii(String html)
    {
        // Convert ASCII html entity sequence
        int lastPos = 0;
        int curPos = html.indexOf("&#");
        while (curPos >= lastPos)
        {
            int offSet = curPos+2;
            while((offSet < html.length())
            && (Character.isDigit(html.charAt(offSet))))
            {
                offSet++;
            }
            if((offSet >= html.length())
            || (html.charAt(offSet) != ';'))
            {
                curPos+=2;
            }
            else
            {
                int ascii = -1;
                try
                {
                    ascii = Integer.parseInt(html.substring(curPos+2,offSet));
                }
                catch (Exception e) { }
                html=html.substring(0, curPos)+((char)ascii)+
                    html.substring(offSet+1);
            }
            lastPos=curPos;
            curPos+=html.substring(curPos).indexOf("&#");
        }
        // Convert other html entity sequences
        for(int i=0; i < ENTITY_LIST.length; i++)
        {
            int length = ENTITY_LIST[i][0].length();
            while ((curPos = html.indexOf(ENTITY_LIST[i][0])) >= 0)
            {
                html = html.substring(0, curPos) + ENTITY_LIST[i][2] +
                    html.substring(curPos+length);
            }
        }
        // Remove embedded blanks (shouldn't be any, but someone is using them)
        while ((curPos = html.indexOf(" ")) >= 0)
        {
            html = html.substring(0,curPos) + html.substring(curPos+1);
        }
        return html;
    }
    /**
    * Convert an input string by remove "^M" from "^M\n" and change to "\n".
    *
    * @param  inStr input string for conversion
    *
    * @return  a string with unix line separator "\n".
    */
    public static String ToUnixLineSeparator(String inStr)
    {
        String outStr = inStr;
        String src = "\n";
        String tar = "\n";
        int tarSize = tar.length();
        int srcSize = src.length();
        int index = outStr.indexOf(src);
        while(index >= 0)
        {
            char mC = outStr.charAt(index-1);
            if(Character.getType(mC) == Character.CONTROL)
            {
                outStr = outStr.substring(0, index-1) + tar
                    + outStr.substring(index+1);
                index = outStr.indexOf(src, index);
            }
            else
            {
                index = outStr.indexOf(src, index+1);
            }
        }
        return outStr;
    }
    /**
    * Convert an input string by add "^M" to "\n" and change to "^M\n".
    *
    * @param  inStr input string for conversion
    *
    * @return  a string with PC line separator "^M\n".
    */
    public static String ToPcLineSeparator(String inStr)
    {
        // convert to Unix first, remove all ^M
        String unixStr = ToUnixLineSeparator(inStr);
        // convert to Pc, add ^M after \n
        String pcStr = Replace(unixStr, UNIX_LINE_SEP, PC_LINE_SEP);
        return pcStr;
    }
    /**
    * Replace a pattern in a String to a specified pattern.
    *
    * @param  inStr   input string for the replacement
    * @param  src     pattern for matching
    * @param  tar     pattern to be replace
    *
    * @return  the new string after the replacement
    */
    public static String Replace(String inStr, String src, String tar)
    {
        String outStr = inStr;
        int tarSize = tar.length();
        int srcSize = src.length();
        int index = outStr.indexOf(src);
        while(index >= 0)
        {
            outStr = outStr.substring(0, index) + tar
                + outStr.substring(index+srcSize);
            index = outStr.indexOf(src, index+tarSize);
        }
        return outStr;
    }
    /** unix line separator */
    final public static String UNIX_LINE_SEP = "\n";
    /** PC line separator */
    final public static String PC_LINE_SEP = "^M\n";
    /** entity list for conversion */
    final private static String[][] ENTITY_LIST = new String[][]
    { 
        {"&#038;", "&amp;", "&"},
        {"&#034;", "&quot;", "\""},
        {"&#060;", "&lt;", "<"},
        {"&#062;", "&gt;", ">"},
        {"&#160;", "&nbsp;", "\u00a0"},
    };
}
